#include <iostream>
#include <sstream>

#include <TROOT.h>
#include <TApplication.h>
#include <TNtupleD.h>
#include <TFile.h>
#include <TMath.h>

#include "MediumMagboltz.hh"
#include "SolidBox.hh"
#include "GeometrySimple.hh"
#include "ComponentConstant.hh"
#include "Sensor.hh"
#include "TrackHeed.hh"
#include "Plotting.hh"
#include "Random.hh"

using namespace Garfield;

int main(int argc, char * argv[]) {

  randomEngine.Seed(123456);
  TApplication app("app", &argc, argv);
  plottingEngine.SetDefaultStyle();

  //--
  // Make a medium
  //--
  MediumMagboltz* gas = new MediumMagboltz();
  gas->SetComposition("ar", 95., "c4h10", 2., "cf4", 3.); // T2K gas
  gas->SetTemperature(293.15); // [K]   NTP
  gas->SetPressure(760.);      // [mHg] NTP

  //--
  // Detector geometry
  //--
  const double width = 0.536; // [cm] pad row pitch of LP1 Asian GEM module
  int    nrows   = 1;             // track length in units of # pad rows
  //int    nrows   = 5;             // track length in units of # pad rows
  double x_c     = 0.;            // [cm]
  double y_c     = 0.;            // [cm]
  double z_c     = 0.;            // [cm]
  double lhalf_x = 10.;           // [cm]
  double lhalf_y = nrows*width/2; // [cm]
  double lhalf_z = 10.;           // [cm]

  SolidBox* box = new SolidBox(x_c, y_c, z_c, lhalf_x, lhalf_y, lhalf_z);
  GeometrySimple* geo = new GeometrySimple();
  geo->AddSolid(box, gas);

  //--
  // Make a component
  //--
  ComponentConstant* comp = new ComponentConstant();
  comp->SetGeometry(geo);
  comp->SetElectricField(0., 0., 230.); // [V/cm]

  //--
  // Make a sensor
  //--
  Sensor* sensor = new Sensor();
  sensor->AddComponent(comp);

  //--
  // Track class
  //--
  TrackHeed* track = new TrackHeed();
  track->SetSensor(sensor);
#if 1
  std::string pid("electron");
  track->SetParticle(pid.data());	// set PID
  track->SetMomentum(5.e9); 		// [eV]
#else
  std::string pid("pi");
  track->SetParticle(pid.data());	// set PID
  track->SetMomentum(0.56e9);		// [eV]
#endif
  // Initial position and direction 
  const double kToRad = TMath::Pi()/180.; // deg to rad conversion factor
  // track angle
  double thetadeg = 0.; // track angle to pad plane
  double phideg   = 0.; // track angle to y axis
  double theta    = thetadeg * kToRad;
  double phi      = phideg   * kToRad;
  // track direction vector
  double dx0   = cos(theta)*sin(phi);
  double dy0   = cos(theta)*cos(phi);
  double dz0   = sin(theta);
  // initial track position 
  double s0    = -lhalf_y/cos(phi)/cos(theta);
  double x0    = x_c + s0 * dx0;
  double y0    = y_c + s0 * dy0;
  double z0    = z_c + s0 * dz0;
  double t0    = 0.;

  //--
  // Output file
  //--
  std::stringstream foutstr;
  foutstr << "edep" << "." << pid.data() 
                    << "." << "phi"   << phideg    << "deg"
                    << "." << "theta" << thetadeg  << "deg"
                    << "." << track->GetMomentum() << ".root"
                    << std::ends;
  TFile *hfp = new TFile(foutstr.str().data(),"RECREATE","Marilin");

  // N-tuples
  static TNtupleD *hHits = 0;
  if (!hHits) {
    std::stringstream list;
    list << "nsum:esum:np:k2sum" << std::ends;
    hHits = new TNtupleD("hHits","",list.str().data());
  }
  static const Int_t kNdata = 10000;
  Double_t data[kNdata];

  const int nEvents = 100000;
  track->EnableDebugging();
  for (int i = 0; i < nEvents; ++i) {
    if (i == 1) track->DisableDebugging();
    if (i % 1000 == 0) std::cout << i << "/" << nEvents << "\n";
    track->NewTrack(x0, y0, z0, t0, dx0, dy0, dz0);
    // Cluster coordinates
    double xc = 0., yc = 0., zc = 0., tc = 0.;
    // Number of electrons produced in a collision
    int nc = 0;
    // Energy loss in a collision
    double ec = 0.;
    // Dummy variable (not used at present)
    double extra = 0.;
    // Total energy loss along the track
    double esum = 0.;
    // Total number of electrons produced along the track
    int nsum  = 0;
    int np    = 0;
    int k2sum = 0;
    // Loop over the clusters.
    while (track->GetCluster(xc, yc, zc, tc, nc, ec, extra)) {
      esum  += ec;
      nsum  += nc;
      k2sum += nc*nc;
      np++;
    }
    data[0] = nsum;
    data[1] = esum*1.e-3; // [keV]
    data[2] = np;
    data[3] = k2sum;
    hHits->Fill(data);
  }

  hfp->Write();

  //app.Run(kTRUE); 
}
